# Copyright 2023 Zurich Instruments AG
# SPDX-License-Identifier: Apache-2.0


from typing import Any as AnyDSL

from laboneq.core.types.enums.averaging_mode import AveragingMode as AveragingModeDSL
from laboneq.core.types.enums.execution_type import ExecutionType as ExecutionTypeDSL
from laboneq.core.types.enums.repetition_mode import RepetitionMode as RepetitionModeDSL
from laboneq.core.types.enums.section_alignment import (
    SectionAlignment as SectionAlignmentDSL,
)
from laboneq.data.calibration import SignalCalibration as SignalCalibrationDATA
from laboneq.data.experiment_description import Acquire as AcquireDATA
from laboneq.data.experiment_description import AcquireLoopNt as AcquireLoopNtDATA
from laboneq.data.experiment_description import AcquireLoopRt as AcquireLoopRtDATA
from laboneq.data.experiment_description import Any as AnyDATA
from laboneq.data.experiment_description import AveragingMode as AveragingModeDATA
from laboneq.data.experiment_description import Call as CallDATA
from laboneq.data.experiment_description import Case as CaseDATA
from laboneq.data.experiment_description import Delay as DelayDATA
from laboneq.data.experiment_description import ExecutionType as ExecutionTypeDATA
from laboneq.data.experiment_description import Experiment as ExperimentDATA
from laboneq.data.experiment_description import ExperimentSignal as ExperimentSignalDATA
from laboneq.data.experiment_description import Match as MatchDATA
from laboneq.data.experiment_description import Operation as OperationDATA
from laboneq.data.experiment_description import Optional as OptionalDATA
from laboneq.data.experiment_description import PlayPulse as PlayPulseDATA
from laboneq.data.experiment_description import Pulse as PulseDATA
from laboneq.data.experiment_description import PulseFunctional as PulseFunctionalDATA
from laboneq.data.experiment_description import (
    PulseSampledComplex as PulseSampledComplexDATA,
)
from laboneq.data.experiment_description import PulseSampledReal as PulseSampledRealDATA
from laboneq.data.experiment_description import RepetitionMode as RepetitionModeDATA
from laboneq.data.experiment_description import Reserve as ReserveDATA
from laboneq.data.experiment_description import Section as SectionDATA
from laboneq.data.experiment_description import SectionAlignment as SectionAlignmentDATA
from laboneq.data.experiment_description import Set as SetDATA
from laboneq.data.experiment_description import Sweep as SweepDATA
from laboneq.data.parameter import LinearSweepParameter as LinearSweepParameterDATA
from laboneq.data.parameter import Parameter as ParameterDATA
from laboneq.data.parameter import SweepParameter as SweepParameterDATA
from laboneq.dsl.calibration.signal_calibration import (
    SignalCalibration as SignalCalibrationDSL,
)
from laboneq.dsl.experiment.acquire import Acquire as AcquireDSL
from laboneq.dsl.experiment.call import Call as CallDSL
from laboneq.dsl.experiment.delay import Delay as DelayDSL
from laboneq.dsl.experiment.experiment import Experiment as ExperimentDSL
from laboneq.dsl.experiment.experiment_signal import (
    ExperimentSignal as ExperimentSignalDSL,
)
from laboneq.dsl.experiment.operation import Operation as OperationDSL
from laboneq.dsl.experiment.play_pulse import PlayPulse as PlayPulseDSL
from laboneq.dsl.experiment.pulse import Pulse as PulseDSL
from laboneq.dsl.experiment.pulse import PulseFunctional as PulseFunctionalDSL
from laboneq.dsl.experiment.pulse import PulseSampledComplex as PulseSampledComplexDSL
from laboneq.dsl.experiment.pulse import PulseSampledReal as PulseSampledRealDSL
from laboneq.dsl.experiment.reserve import Reserve as ReserveDSL
from laboneq.dsl.experiment.section import AcquireLoopNt as AcquireLoopNtDSL
from laboneq.dsl.experiment.section import AcquireLoopRt as AcquireLoopRtDSL
from laboneq.dsl.experiment.section import Case as CaseDSL
from laboneq.dsl.experiment.section import Match as MatchDSL
from laboneq.dsl.experiment.section import Section as SectionDSL
from laboneq.dsl.experiment.section import Sweep as SweepDSL
from laboneq.dsl.experiment.set_node import SetNode as SetDSL
from laboneq.dsl.parameter import LinearSweepParameter as LinearSweepParameterDSL
from laboneq.dsl.parameter import Parameter as ParameterDSL
from laboneq.dsl.parameter import SweepParameter as SweepParameterDSL
from laboneq.implementation.legacy_adapters.dynamic_converter import convert_dynamic

# converter functions for data type package 'experiment_description'
#  AUTOGENERATED, DO NOT EDIT
from .post_process_experiment_description import post_process


def get_converter_function_experiment_description(orig):
    converter_function_directory = {
        AcquireDSL: convert_Acquire,
        AcquireLoopNtDSL: convert_AcquireLoopNt,
        AcquireLoopRtDSL: convert_AcquireLoopRt,
        CallDSL: convert_Call,
        CaseDSL: convert_Case,
        DelayDSL: convert_Delay,
        ExperimentDSL: convert_Experiment,
        ExperimentSignalDSL: convert_ExperimentSignal,
        LinearSweepParameterDSL: convert_LinearSweepParameter,
        MatchDSL: convert_Match,
        OperationDSL: convert_Operation,
        ParameterDSL: convert_Parameter,
        PlayPulseDSL: convert_PlayPulse,
        PulseFunctionalDSL: convert_PulseFunctional,
        PulseSampledComplexDSL: convert_PulseSampledComplex,
        PulseSampledRealDSL: convert_PulseSampledReal,
        ReserveDSL: convert_Reserve,
        SectionDSL: convert_Section,
        SetDSL: convert_Set,
        SignalCalibrationDSL: convert_SignalCalibration,
        SweepDSL: convert_Sweep,
        SweepParameterDSL: convert_SweepParameter,
    }
    return converter_function_directory.get(orig)


def convert_AveragingMode(orig: AveragingModeDSL):
    return (
        next(e for e in AveragingModeDATA if e.name == orig.name)
        if orig is not None
        else None
    )


def convert_ExecutionType(orig: ExecutionTypeDSL):
    return (
        next(e for e in ExecutionTypeDATA if e.name == orig.name)
        if orig is not None
        else None
    )


def convert_RepetitionMode(orig: RepetitionModeDSL):
    return (
        next(e for e in RepetitionModeDATA if e.name == orig.name)
        if orig is not None
        else None
    )


def convert_SectionAlignment(orig: SectionAlignmentDSL):
    return (
        next(e for e in SectionAlignmentDATA if e.name == orig.name)
        if orig is not None
        else None
    )


def convert_Acquire(orig: AcquireDSL):
    if orig is None:
        return None
    retval = AcquireDATA()
    retval.handle = orig.handle
    retval.kernel = convert_dynamic(
        orig.kernel,
        source_type_hint=PulseDSL,
        target_type_hint=PulseDATA,
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.length = orig.length
    retval.pulse_parameters = convert_dynamic(
        orig.pulse_parameters,
        source_type_string="Dict",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.signal = orig.signal
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_AcquireLoopNt(orig: AcquireLoopNtDSL):
    if orig is None:
        return None
    retval = AcquireLoopNtDATA()
    retval.averaging_mode = convert_AveragingMode(orig.averaging_mode)
    retval.count = orig.count
    retval.execution_type = convert_ExecutionType(orig.execution_type)
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_AcquireLoopRt(orig: AcquireLoopRtDSL):
    if orig is None:
        return None
    retval = AcquireLoopRtDATA()
    retval.acquisition_type = orig.acquisition_type
    retval.averaging_mode = convert_AveragingMode(orig.averaging_mode)
    retval.count = orig.count
    retval.execution_type = convert_ExecutionType(orig.execution_type)
    retval.repetition_mode = convert_RepetitionMode(orig.repetition_mode)
    retval.repetition_time = orig.repetition_time
    retval.reset_oscillator_phase = orig.reset_oscillator_phase
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Call(orig: CallDSL):
    if orig is None:
        return None
    retval = CallDATA()
    retval.args = convert_dynamic(
        orig.args,
        source_type_string="Dict",
        target_type_string="Dict",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.func_name = convert_dynamic(
        orig.func_name,
        source_type_hint=AnyDSL,
        target_type_hint=AnyDATA,
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Case(orig: CaseDSL):
    if orig is None:
        return None
    retval = CaseDATA()
    retval.state = orig.state
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Delay(orig: DelayDSL):
    if orig is None:
        return None
    retval = DelayDATA()
    retval.precompensation_clear = orig.precompensation_clear
    retval.signal = orig.signal
    retval.time = convert_dynamic(
        orig.time,
        source_type_string="Union[float, Parameter]",
        target_type_string="Parameter",
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Experiment(orig: ExperimentDSL):
    if orig is None:
        return None
    retval = ExperimentDATA()
    retval._section_stack = convert_dynamic(
        orig._section_stack,
        source_type_hint=SectionDSL,
        target_type_hint=SectionDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.epsilon = orig.epsilon
    retval.sections = convert_dynamic(
        orig.sections,
        source_type_hint=SectionDSL,
        target_type_hint=SectionDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.signals = convert_dynamic(
        orig.signals,
        source_type_string="Union[Dict[str, ExperimentSignal], List[ExperimentSignal]]",
        target_type_string="Union[Dict[str, ExperimentSignal], List[ExperimentSignal]]",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_ExperimentSignal(orig: ExperimentSignalDSL):
    if orig is None:
        return None
    retval = ExperimentSignalDATA()
    retval.calibration = convert_SignalCalibration(orig.calibration)
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_LinearSweepParameter(orig: LinearSweepParameterDSL):
    if orig is None:
        return None
    retval = LinearSweepParameterDATA()
    retval.axis_name = orig.axis_name
    retval.count = orig.count
    retval.start = orig.start
    retval.stop = orig.stop
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Match(orig: MatchDSL):
    if orig is None:
        return None
    retval = MatchDATA()
    retval.handle = orig.handle
    retval.user_register = orig.user_register
    retval.local = orig.local
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Operation(orig: OperationDSL):
    if orig is None:
        return None
    retval = OperationDATA()
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Parameter(orig: ParameterDSL):
    if orig is None:
        return None
    retval = ParameterDATA()
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_PlayPulse(orig: PlayPulseDSL):
    if orig is None:
        return None
    retval = PlayPulseDATA()
    retval.amplitude = convert_dynamic(
        orig.amplitude,
        source_type_string="Union[float, complex, Parameter]",
        target_type_string="Union[float, complex, Parameter]",
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.increment_oscillator_phase = convert_dynamic(
        orig.increment_oscillator_phase,
        source_type_string="Union[float, Parameter]",
        target_type_string="Parameter",
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.length = convert_dynamic(
        orig.length,
        source_type_string="Union[float, Parameter]",
        target_type_string="Parameter",
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.marker = convert_dynamic(
        orig.marker,
        source_type_string="Dict",
        target_type_string="Dict",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.phase = orig.phase
    retval.precompensation_clear = orig.precompensation_clear
    retval.pulse = convert_dynamic(
        orig.pulse,
        source_type_hint=PulseDSL,
        target_type_hint=PulseDATA,
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.pulse_parameters = convert_dynamic(
        orig.pulse_parameters,
        source_type_hint=AnyDSL,
        target_type_hint=OptionalDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.set_oscillator_phase = orig.set_oscillator_phase
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_PulseFunctional(orig: PulseFunctionalDSL):
    if orig is None:
        return None
    retval = PulseFunctionalDATA()
    retval.amplitude = orig.amplitude
    retval.function = orig.function
    retval.length = orig.length
    retval.pulse_parameters = convert_dynamic(
        orig.pulse_parameters,
        source_type_hint=AnyDSL,
        target_type_hint=OptionalDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_PulseSampledComplex(orig: PulseSampledComplexDSL):
    if orig is None:
        return None
    retval = PulseSampledComplexDATA()
    retval.samples = convert_dynamic(
        orig.samples,
        source_type_string="ArrayLike",
        target_type_string="ArrayLike",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_PulseSampledReal(orig: PulseSampledRealDSL):
    if orig is None:
        return None
    retval = PulseSampledRealDATA()
    retval.samples = convert_dynamic(
        orig.samples,
        source_type_string="ArrayLike",
        target_type_string="ArrayLike",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Reserve(orig: ReserveDSL):
    if orig is None:
        return None
    retval = ReserveDATA()
    retval.signal = orig.signal
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Section(orig: SectionDSL):
    if orig is None:
        return None
    retval = SectionDATA()
    retval.alignment = convert_SectionAlignment(orig.alignment)
    retval.children = convert_dynamic(
        orig.children,
        source_type_hint=OperationDSL,
        target_type_hint=OperationDATA,
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.execution_type = convert_ExecutionType(orig.execution_type)
    retval.length = orig.length
    retval.on_system_grid = orig.on_system_grid
    retval.play_after = convert_dynamic(
        orig.play_after,
        source_type_string="Union[str, List[str]]",
        target_type_string="List[str]",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.trigger = convert_dynamic(
        orig.trigger,
        source_type_string="Dict[str, Dict]",
        target_type_string="Dict",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Set(orig: SetDSL):
    if orig is None:
        return None
    retval = SetDATA()
    retval.key = orig.key
    retval.path = orig.path
    retval.value = convert_dynamic(
        orig.value,
        source_type_hint=AnyDSL,
        target_type_hint=AnyDATA,
        orig_is_collection=False,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_SignalCalibration(orig: SignalCalibrationDSL):
    if orig is None:
        return None
    retval = SignalCalibrationDATA()
    retval.amplitude = convert_dynamic(
        orig.amplitude,
        source_type_string="float | Parameter | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.delay_signal = convert_dynamic(
        orig.delay_signal,
        source_type_string="float | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.local_oscillator = convert_dynamic(
        orig.local_oscillator,
        source_type_string="Oscillator | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.mixer_calibration = convert_dynamic(
        orig.mixer_calibration,
        source_type_string="MixerCalibration | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.oscillator = convert_dynamic(
        orig.oscillator,
        source_type_string="Oscillator | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.port_delay = convert_dynamic(
        orig.port_delay,
        source_type_string="float | Parameter | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.port_mode = convert_dynamic(
        orig.port_mode,
        source_type_string="PortMode | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.precompensation = convert_dynamic(
        orig.precompensation,
        source_type_string="Precompensation | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.range = convert_dynamic(
        orig.range,
        source_type_string="int | float | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.threshold = convert_dynamic(
        orig.threshold,
        source_type_string="float | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.voltage_offset = convert_dynamic(
        orig.voltage_offset,
        source_type_string="float | None",
        target_type_string="Any",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_Sweep(orig: SweepDSL):
    if orig is None:
        return None
    retval = SweepDATA()
    retval.execution_type = convert_ExecutionType(orig.execution_type)
    retval.parameters = convert_dynamic(
        orig.parameters,
        source_type_hint=ParameterDSL,
        target_type_hint=ParameterDATA,
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    retval.reset_oscillator_phase = orig.reset_oscillator_phase
    retval.uid = orig.uid
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )


def convert_SweepParameter(orig: SweepParameterDSL):
    if orig is None:
        return None
    retval = SweepParameterDATA()
    retval.axis_name = orig.axis_name
    retval.uid = orig.uid
    retval.values = convert_dynamic(
        orig.values,
        source_type_string="ArrayLike",
        target_type_string="ArrayLike",
        orig_is_collection=True,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
    return post_process(
        orig,
        retval,
        conversion_function_lookup=get_converter_function_experiment_description,
    )
